@@ -18,14 +18,14 @@ class PowerlineSetAction(PlayableAction):
     else will have no impact.
 
     """
-    def __init__(self, gridobj):
-        super().__init__(gridobj)
+    authorized_keys = {
+        "set_line_status",
+        "set_bus"
+    }
+    attr_list_vect = [
+        "_set_line_status"
+    ]
+    attr_list_set = set(attr_list_vect)
 
-        self.authorized_keys = {
-            "set_line_status",
-            "set_bus"
-        }
-
-        self.attr_list_vect = [
-            "_set_line_status"
-        ]
+    def __init__(self):
+        super().__init__()
