@@ -7,6 +7,8 @@
 # This file is part of Grid2Op, Grid2Op a testbed platform to model sequential decision making in power systems.
 
 import os
+import json
+import warnings
 import numpy as np
 from datetime import timedelta, datetime
 
@@ -101,10 +103,43 @@ class Multifolder(GridValue):
         """
         Assign a filtering function to remove some chronics from the next time a call to "reset_cache" is called.
 
-        **NB** filter_fun is applied to all element of :attr:`MultifolderWithCache.subpaths`. If ``True`` then it will
+        **NB** filter_fun is applied to all element of :attr:`Multifolder.subpaths`. If ``True`` then it will
         be put in cache, if ``False`` this data will NOT be put in the cache.
 
         **NB** this has no effect until :attr:`Multifolder.reset` is called.
+
+        Examples
+        --------
+        Let's assume in your chronics, the folder names are "Scenario_august_dummy", and
+        "Scenario_february_dummy". For the sake of the example, we want the environment to loop
+        only through the month of february, because why not. Then we can do the following:
+
+        .. code-block:: python
+
+            import re
+            import grid2op
+            env = grid2op.make("l2rpn_neurips_2020_track1", test=True)  # don't add "test=True" if
+            # you don't want to perform a test.
+
+            # check at which month will belong each observation
+            for i in range(10):
+                obs = env.reset()
+                print(obs.month)
+                # it always alternatively prints "8" (if chronics if from august) or
+                # "2" if chronics is from february)
+
+            # to see where the chronics are located
+            print(env.chronics_handler.subpaths)
+
+            # keep only the month of february
+            env.chronics_handler.set_filter(lambda path: re.match(".*february.*", path) is not None)
+            env.chronics_handler.reset()  # if you don't do that it will not have any effect
+
+            for i in range(10):
+                obs = env.reset()
+                print(obs.month)
+                # it always prints "2" (representing february)
+
         """
         self._filter = filter_fun
 
@@ -124,20 +159,49 @@ class Multifolder(GridValue):
         -----------
         probabilities: ``np.ndarray``
             Array of integer with the same size as the number of chronics in the cache.
+            If it does not sum to one, it is rescaled such that it sums to one.
 
         Returns
         -------
         selected: ``int``
             The integer that was selected.
 
+        Examples
+        --------
+
+        Let's assume in your chronics, the folder names are "Scenario_august_dummy", and
+        "Scenario_february_dummy". For the sake of the example, we want the environment to loop
+        75% of the time to the month of february and 25% of the time to the month of august.
+
+        .. code-block:: python
+
+            import grid2op
+            env = grid2op.make("l2rpn_neurips_2020_track1", test=True)  # don't add "test=True" if
+            # you don't want to perform a test.
+
+            # check at which month will belong each observation
+            for i in range(10):
+                obs = env.reset()
+                print(obs.month)
+                # it always alternatively prints "8" (if chronics if from august) or
+                # "2" if chronics is from february) with a probability of 50% / 50%
+
+            env.seed(0)  # for reproducible experiment
+            for i in range(10):
+                _ = env.chronics_handler.sample_next_chronics([0.25, 0.75])
+                obs = env.reset()
+                print(obs.month)
+                # it prints "2" with probability 0.75 and "8" with probability 0.25
+
         """
         self._prev_cache_id = -1
         # make sure it sums to 1
         probabilities /= np.sum(probabilities)
         # take one at "random" among these
         selected = self.space_prng.choice(self._order,  p=probabilities)
+        id_sel = np.where(self._order == selected)[0]
         self._prev_cache_id = selected - 1
-        return selected
+        return id_sel
 
     def reset(self):
         """
@@ -357,7 +421,7 @@ class Multifolder(GridValue):
             for subpath in self.subpaths:
                 id_this_chron = os.path.split(subpath)[-1]
                 datetime_end[id_this_chron] = datetime_end_orig
-
+        seed_chronics_all = {}
         for subpath in self.subpaths:
             id_this_chron = os.path.split(subpath)[-1]
             if not id_this_chron in datetime_beg:
@@ -367,6 +431,12 @@ class Multifolder(GridValue):
                                       path=subpath,
                                       max_iter=self.max_iter,
                                       chunk_size=self.chunk_size)
+            seed_chronics = None
+            if self.seed is not None:
+                max_int = np.iinfo(dt_int).max
+                seed_chronics = self.space_prng.randint(max_int)
+                tmp.seed(seed_chronics)
+            seed_chronics_all[subpath] = seed_chronics
             tmp.initialize(self._order_backend_loads,
                            self._order_backend_prods,
                            self._order_backend_lines,
@@ -375,6 +445,21 @@ class Multifolder(GridValue):
             path_out_chron = os.path.join(path_out, id_this_chron)
             tmp.split_and_save(datetime_beg[id_this_chron], datetime_end[id_this_chron], path_out_chron)
 
+            meta_params = {}
+            meta_params["datetime_beg"] = datetime_beg
+            meta_params["datetime_end"] = datetime_end
+            meta_params["path_out"] = path_out
+            meta_params["all_seeds"] = seed_chronics_all
+            try:
+                with open(os.path.join(path_out, "split_and_save_meta_params.json"), "w", encoding="utf-8") as f:
+                    json.dump(obj=meta_params, fp=f,
+                              sort_keys=True,
+                              indent=4
+                              )
+            except Exception as exc_:
+                warnings.warn("Impossible to save the \"metadata\" for the chronics with error:\n\"{}\""
+                              "".format(exc_))
+
     def fast_forward(self, nb_timestep):
         """
         This method allows you to skip some time step at the beginning of the chronics.
