@@ -166,7 +166,8 @@ class Environment(BaseEnv):
         if _raw_backend_class is None:
             self._raw_backend_class = type(backend)
         else:
-            _raw_backend_class = _raw_backend_class
+            self._raw_backend_class = _raw_backend_class
+
         # for plotting
         self.init_backend(init_grid_path, chronics_handler, backend,
                           names_chronics_to_backend, actionClass, observationClass,
@@ -213,15 +214,15 @@ class Environment(BaseEnv):
             raise Grid2OpException( "Parameter \"backend\" used to build the Environment should derived form the "
                                     "grid2op.Backend class, type provided is \"{}\"".format(type(backend)))
         self.backend = backend
+        # all the above should be done in this exact order, otherwise some weird behaviour might occur
+        # this is due to the class attribute
+        self.backend.set_env_name(self.name)
         self.backend.load_grid(self.init_grid_path)  # the real powergrid of the environment
-
         self.backend.load_redispacthing_data(os.path.split(self.init_grid_path)[0])
         self.backend.load_grid_layout(os.path.split(self.init_grid_path)[0])
-        self.backend.set_env_name(self.name)
-
         self.backend.assert_grid_correct()
-
-        self._has_been_initialized()  # really important to include this piece of code!
+        self._has_been_initialized()  # really important to include this piece of code! and just here after the
+        # backend has loaded everything
 
         if self._thermal_limit_a is None:
             self._thermal_limit_a = self.backend.thermal_limit_a.astype(dt_float)
@@ -426,7 +427,7 @@ class Environment(BaseEnv):
             from grid2op import make
             from grid2op.BaseAgent import DoNothingAgent
 
-            env = make("case14_redisp")  # create an environment
+            env = make("rte_case14_realistic")  # create an environment
             agent = DoNothingAgent(env.action_space)  # create an BaseAgent
 
             for i in range(10):
@@ -438,7 +439,33 @@ class Environment(BaseEnv):
                     act = agent.act(obs, reward, done)
                     obs, reward, done, info = env.step(act)
 
+        And here you have an example on how you can loop through the scenarios in a given order:
+
+        .. code-block:: python
+
+            import grid2op
+            from grid2op import make
+            from grid2op.BaseAgent import DoNothingAgent
+
+            env = make("rte_case14_realistic")  # create an environment
+            agent = DoNothingAgent(env.action_space)  # create an BaseAgent
+            scenario_order = [1,2,3,4,5,10,8,6,5,7,78, 8]
+            for id_ in scenario_order:
+                env.set_id(id_)  # tell the environment you simply want to use the chronics with ID 0
+                obs = env.reset()  # it is necessary to perform a reset
+                reward = env.reward_range[0]
+                done = False
+                while not done:
+                    act = agent.act(obs, reward, done)
+                    obs, reward, done, info = env.step(act)
+
         """
+        try:
+            id_ = int(id_)
+        except:
+            raise EnvError("the \"id_\" parameters should be convertible to integer and not be of type {}"
+                           "".format(type(id_)))
+
         self.chronics_handler.tell_id(id_-1)
 
     def attach_renderer(self, graph_layout=None):
@@ -579,11 +606,31 @@ class Environment(BaseEnv):
         """
         tmp_backend = self.backend
         self.backend = None
+
+        tmp_obs_space = self.helper_observation
+        self.observation_space = None
+        self.helper_observation = None
+
+        obs_tmp = self.current_obs
+        self.current_obs = None
+
+        volt_cont = self.voltage_controler
+        self.voltage_controler = None
+
         res = copy.deepcopy(self)
         res.backend = tmp_backend.copy()
+        res.helper_observation = tmp_obs_space.copy()
+        res.observation_space = res.helper_observation
+        res.current_obs = obs_tmp.copy()
+        res.voltage_controler = volt_cont.copy()
+
         if self._thermal_limit_a is not None:
             res.backend.set_thermal_limit(self._thermal_limit_a)
         self.backend = tmp_backend
+        self.observation_space = tmp_obs_space
+        self.helper_observation = tmp_obs_space
+        self.current_obs = obs_tmp
+        self.voltage_controler = volt_cont
         return res
 
     def get_kwargs(self, with_backend=True):
