@@ -110,7 +110,31 @@ class EpisodeData:
                                               "observations")
         self.env_actions = CollectionWrapper(env_actions,
                                              helper_action_env,
-                                             "env_actions")
+                                             "env_actions",
+                                             check_legit=False)
+        # gives a unique game over for everyone
+        # TODO this needs testing!
+        action_go = self.actions._game_over
+        obs_go = self.observations._game_over
+        env_go = self.env_actions._game_over
+        real_go = action_go
+        if real_go is None:
+            real_go = obs_go
+        else:
+            if obs_go is not None:
+                real_go = min(obs_go, real_go)
+        if real_go is None:
+            real_go = env_go
+        else:
+            if env_go is not None:
+                real_go = min(env_go, real_go)
+        if real_go is not None:
+            # there is a real game over, i assign the proper value for each collection
+            self.actions._game_over = real_go
+            self.observations._game_over = real_go + 1
+            self.env_actions._game_over = real_go
+
+        self.attack = attack
         self.other_rewards = other_rewards
         self.observation_space = observation_space
         self.rewards = rewards
@@ -123,7 +147,6 @@ class EpisodeData:
         self.disc_lines_templ = disc_lines_templ
 
         self.attack_templ = attack_templ
-        self.attack = attack
         self.attack_space = attack_space
 
         self.logger = logger
@@ -196,7 +219,7 @@ class EpisodeData:
     def from_disk(cls, agent_path, name=str(1)):
 
         if agent_path is None:
-            raise Grid2OpException("A path to an episode should be provided, please call \"from_disck\" with "
+            raise Grid2OpException("A path to an episode should be provided, please call \"from_disk\" with "
                                    "\"agent_path other\" than None")
         episode_path = os.path.abspath(os.path.join(agent_path, name))
 
@@ -214,7 +237,7 @@ class EpisodeData:
                 episode_path, EpisodeData.AG_EXEC_TIMES))["data"]
             actions = np.load(os.path.join(episode_path, EpisodeData.ACTIONS))["data"]
             env_actions = np.load(os.path.join(
-                episode_path,EpisodeData.ENV_ACTIONS))["data"]
+                episode_path, EpisodeData.ENV_ACTIONS))["data"]
             observations = np.load(os.path.join(
                 episode_path, EpisodeData.OBSERVATIONS))["data"]
             disc_lines = np.load(os.path.join(
@@ -247,7 +270,7 @@ class EpisodeData:
                    observation_space=observation_space,
                    action_space=action_space,
                    helper_action_env=helper_action_env,
-                   path_save=agent_path,
+                   path_save=None, # No save when reading
                    attack=attack,
                    attack_space=attack_space,
                    name=name,
@@ -435,7 +458,7 @@ class CollectionWrapper:
 
     """
 
-    def __init__(self, collection, helper, collection_name):
+    def __init__(self, collection, helper, collection_name, check_legit=True):
         self.collection = collection
         if not hasattr(helper, "from_vect"):
             raise Grid2OpException(f"Object {helper} must implement a "
@@ -448,8 +471,9 @@ class CollectionWrapper:
         self.objects = []
         for i, elem in enumerate(self.collection):
             try:
-                self.objects.append(
-                    self.helper.from_vect(self.collection[i, :]))
+                collection_obj = self.helper.from_vect(self.collection[i, :],
+                                                       check_legit=check_legit)
+                self.objects.append(collection_obj)
             except AmbiguousAction:
                 self._game_over = i
                 break
@@ -490,4 +514,4 @@ class CollectionWrapper:
 
 
 if __name__ == "__main__":
-    pass
\ No newline at end of file
+    pass
