@@ -58,6 +58,7 @@ class EpisodeReplay(object):
 
     def replay_episode(self, episode_id, fps=2.0, gif_name=None,
                        display=True, start_step=0, end_step=-1,
+                       line_info="rho", load_info="p", gen_info="p",
                        resolution=(1280, 720)):
         """
         When called, this function will start the display of the episode in a "mini movie" format.
@@ -82,6 +83,18 @@ class EpisodeReplay(object):
             Default to -1. The step at which to stop generating the gif.
             Set to -1 to specify no limit
 
+        load_info: ``str``
+            Defaults to "p". What kind of values to show on loads.
+            Can be oneof `["p", "v", None]`
+
+        gen_info: ``str``
+            Defaults to "p". What kind of values to show on generators.
+            Can be oneof `["p", "v", None]`
+
+        line_info: ``str``
+            Defaults to "rho". What kind of values to show on lines.
+            Can be oneof `["rho", "a", "p", "v", None]` 
+
         resolution: ``tuple``
             Defaults to (1280, 720). The resolution to use for the gif.
         """
@@ -93,7 +106,6 @@ class EpisodeReplay(object):
         # Load episode observations
         self.episode_data = EpisodeData.from_disk(agent_path=self.agent_path, name=episode_id)
         all_obs = [el for el in self.episode_data.observations]
-
         # Create a plotter
         width, height = resolution
         plot_runner = PlotMatplot(self.episode_data.observation_space,
@@ -120,7 +132,12 @@ class EpisodeReplay(object):
             start_time = time.time()
 
             # Render the observation
-            fig = plot_runner.plot_obs(observation=obs, figure=figure, redraw=True)
+            fig = plot_runner.plot_obs(observation=obs,
+                                       line_info=line_info,
+                                       gen_info=gen_info,
+                                       load_info=load_info,
+                                       figure=figure,
+                                       redraw=True)
             if figure is None and display:
                 fig.show()
             elif display:
@@ -163,9 +180,12 @@ def episode_replay_cli():
     parser = argparse.ArgumentParser(description="EpisodeReplay")
     parser.add_argument("--agent_path", required=True, type=str)
     parser.add_argument("--episode_id", required=True, type=str)
-    parser.add_argument("--display", required=False, default=False, action="store_true")
+    parser.add_argument("--display", required=False,
+                        default=False, action="store_true")
     parser.add_argument("--fps", required=False, default=2.0, type=float)
     parser.add_argument("--gif_name", required=False, default=None, type=str)
+    parser.add_argument("--gif_start", required=False, default=0, type=int)
+    parser.add_argument("--gif_end", required=False, default=-1, type=int)
     args = parser.parse_args()
     return args
 
@@ -177,6 +197,8 @@ def main(args=None):
     er.replay_episode(args.episode_id,
                       fps=args.fps,
                       gif_name=args.gif_name,
+                      start_step=args.gif_start,
+                      end_step=args.gif_end,
                       display=args.display)
 
 
