@@ -31,6 +31,7 @@ TEST_DEV_ENVS = {
     "l2rpn_wcci_2020": DEV_DATASET.format("l2rpn_wcci_2020"),
     "rte_case14_opponent": DEV_DATASET.format("rte_case14_opponent"),
     "l2rpn_neurips_2020_track2": DEV_DATASET.format("l2rpn_neurips_2020_track2"),
+    "l2rpn_neurips_2020_track1": DEV_DATASET.format("l2rpn_neurips_2020_track1"),
     # keep the old names for now
     "case14_realistic": DEV_DATASET.format("rte_case14_realistic"),
     "case14_redisp": DEV_DATASET.format("rte_case14_redisp"),
@@ -40,7 +41,8 @@ TEST_DEV_ENVS = {
 }
 
 _REQUEST_FAIL_EXHAUSTED_ERR = "Impossible to retrieve data at \"{}\".\n" \
-                              "If the problem persists, please contact grid2op organizers"
+                              "If the problem persists, please contact grid2op developers by sending an issue at " \
+                              "https://github.com/rte-france/Grid2Op/issues"
 _REQUEST_FAIL_RETRY_ERR = "Failure to get a reponse from the url \"{}\".\n" \
                           "Retrying.. {} attempt(s) remaining"
 _REQUEST_EXCEPT_RETRY_ERR = "Exception in getting an answer from \"{}\".\n" \
@@ -63,6 +65,8 @@ def _send_request_retry(url, nb_retry=10, gh_session=None):
         return _send_request_retry(url, nb_retry=nb_retry-1, gh_session=gh_session)
     except Grid2OpException:
         raise
+    except KeyboardInterrupt:
+        raise
     except:
         warnings.warn(_REQUEST_EXCEPT_RETRY_ERR.format(url, nb_retry-1))
         time.sleep(1)
@@ -82,9 +86,13 @@ _LIST_REMOTE_INVALID_DATASETS_JSON_ERR = "Impossible to retrieve available datas
                                          "File could not be converted to json. " \
                                          "The error was \n\"{}\""
 
+_FETCH_ENV_UNKNOWN_ERR = "Impossible to find the environment named \"{}\".\n" \
+                         "Current available environments are:\n{}"
+# _FETCH_ENV_TAR_URL = "https://github.com/BDonnot/grid2op-datasets/releases/download/{}/{}.tar.bz2"
 
-def _list_available_remote_env_aux():
-    answer = _send_request_retry(_LIST_REMOTE_URL)
+
+def _retrieve_github_content(url, is_json=True):
+    answer = _send_request_retry(url)
     try:
         answer_json = answer.json()
     except Exception as e:
@@ -94,16 +102,18 @@ def _list_available_remote_env_aux():
         raise Grid2OpException(_LIST_REMOTE_CORRUPTED_CONTENT_JSON_ERR)
     time.sleep(1)
     avail_datasets = _send_request_retry(answer_json[_LIST_REMOTE_KEY])
-    try:
-        avail_datasets_json = avail_datasets.json()
-    except Exception as e:
-        raise Grid2OpException(_LIST_REMOTE_INVALID_DATASETS_JSON_ERR.format(e))
-    return avail_datasets_json
+    if is_json:
+        try:
+            res = avail_datasets.json()
+        except Exception as e:
+            raise Grid2OpException(_LIST_REMOTE_INVALID_DATASETS_JSON_ERR.format(e))
+    else:
+        res = avail_datasets.text
+    return res
 
 
-_FETCH_ENV_UNKNOWN_ERR = "Impossible to find the environment named \"{}\".\n" \
-                         "Current available environments are:\n{}"
-# _FETCH_ENV_TAR_URL = "https://github.com/BDonnot/grid2op-datasets/releases/download/{}/{}.tar.bz2"
+def _list_available_remote_env_aux():
+    return _retrieve_github_content(url=_LIST_REMOTE_URL)
 
 
 def _fecth_environments(dataset_name):
@@ -112,7 +122,7 @@ def _fecth_environments(dataset_name):
         known_ds = sorted(avail_datasets_json.keys())
         raise UnknownEnv(_FETCH_ENV_UNKNOWN_ERR.format(dataset_name, known_ds))
     # url = _FETCH_ENV_TAR_URL.format(avail_datasets_json[dataset_name], dataset_name)
-    dict_ =  avail_datasets_json[dataset_name]
+    dict_ = avail_datasets_json[dataset_name]
     baseurl, filename = dict_["base_url"], dict_["filename"]
     url = baseurl + filename
     # name is "tar.bz2" so i need to get rid of 2 extensions
@@ -247,7 +257,7 @@ def make(dataset="rte_case14_realistic", test=False, **kwargs):
 
     # Unknown dev env
     if test and dataset_name not in TEST_DEV_ENVS:
-        raise Grid2OpException(_MAKE_UNKNOWN_ENV)
+        raise Grid2OpException(_MAKE_UNKNOWN_ENV.format(dataset))
 
     # Known test env and test flag enabled
     if test:
@@ -264,6 +274,8 @@ def make(dataset="rte_case14_realistic", test=False, **kwargs):
 
     # Env directory is present in the DEFAULT_PATH_DATA
     if os.path.exists(real_ds_path):
+        if _aux_is_multimix(real_ds_path):
+            make_from_path_fn = _aux_make_multimix
         return make_from_path_fn(real_ds_path, **kwargs)
 
     # Env needs to be downloaded
