@@ -7,6 +7,7 @@
 # This file is part of Grid2Op, Grid2Op a testbed platform to model sequential decision making in power systems.
 
 import numpy as np
+from grid2op.Exceptions import IllegalAction
 from grid2op.Rules.BaseRules import BaseRules
 
 
@@ -27,14 +28,23 @@ class PreventReconnection(BaseRules):
         See :func:`BaseRules.__call__` for a definition of the parameters of this function.
 
         """
-        aff_lines, aff_subs = action.get_topological_impact()
+        # at first iteration, env.current_obs is None...
+        # TODO this is used inside the environment (for step) inside LookParam and here
+        # this could be computed only once, and fed to this instead
+        if env.current_obs is not None:
+            powerline_status = env.current_obs.line_status
+        else:
+            powerline_status = None
+        aff_lines, aff_subs = action.get_topological_impact(powerline_status)
         if np.any(env.times_before_line_status_actionable[aff_lines] > 0):
             # i tried to act on a powerline too shortly after a previous action
             # or shut down due to an overflow or opponent or hazards or maintenance
-            return False
+            ids = np.where(env.times_before_line_status_actionable[aff_lines] > 0)[0]
+            return False, IllegalAction("Powerline with ids {} have been modified illegally (cooldown)".format(ids))
 
         if np.any(env.times_before_topology_actionable[aff_subs] > 0):
             # I tried to act on a topology too shortly after a previous action
-            return False
+            ids = np.where(env.times_before_line_status_actionable[aff_lines] > 0)[0]
+            return False, IllegalAction("Substation with ids {} have been modified illegally (cooldown)".format(ids))
 
-        return True
+        return True, None
