@@ -2,7 +2,7 @@
 Grid2Op
 
 """
-__version__ = '1.0.0'
+__version__ = '1.2.1'
 
 __all__ = [
     "Action",
@@ -28,8 +28,10 @@ __all__ = [
     "list_available_remote_env",
     "list_available_local_env",
     "get_current_local_dir",
-    "change_local_dir"
+    "change_local_dir",
+    "update_env"
 ]
 
 from grid2op.MakeEnv import make_old, make, make_from_dataset_path
+from grid2op.MakeEnv import update_env
 from grid2op.MakeEnv import list_available_remote_env, list_available_local_env, get_current_local_dir, change_local_dir
