@@ -9,7 +9,7 @@
 import pdb
 import warnings
 import pandas as pd
-
+import tempfile
 from grid2op.tests.helper_path_test import *
 
 from grid2op.dtypes import dt_int, dt_float
@@ -21,6 +21,7 @@ from grid2op.Backend import PandaPowerBackend
 from grid2op.Parameters import Parameters
 from grid2op.Rules import AlwaysLegal
 
+
 class TestProperHandlingHazardsMaintenance(HelperTests):
     def setUp(self):
         self.path_hazard = os.path.join(PATH_CHRONICS, "chronics_with_hazards")
@@ -749,6 +750,74 @@ class TestCFFWFWM(HelperTests):
                 assert maxMaintenancePerDay == 0
                 assert np.sum(env.chronics_handler.real_data.data.maintenance) == 0
 
+
+    def test_split_and_save(self):
+        param = Parameters()
+        param.NO_OVERFLOW_DISCONNECTION = True
+        with warnings.catch_warnings():
+            warnings.filterwarnings("ignore")
+            with make(os.path.join(PATH_DATA_TEST, "ieee118_R2subgrid_wcci_test_maintenance"), param=param) as env:
+                env.seed(0)
+                env.set_id(0)
+                obs = env.reset()
+                start_d = {
+                    "Scenario_august_00": "2012-08-10 00:00"
+                }
+                end_d = {
+                    "Scenario_august_00": "2012-08-19 00:00"
+                }
+                ts_beg = 2591
+                ts_end = 5184
+
+                # generate some reference data
+                chronics_outdir = tempfile.mkdtemp()
+                env.chronics_handler.real_data.split_and_save(start_d,
+                                                              end_d,
+                                                              chronics_outdir)
+                maintenance_0_0 = pd.read_csv(os.path.join(chronics_outdir,
+                                                         "Scenario_august_00",
+                                                         "maintenance.csv.bz2"),
+                                            sep=";").values
+
+                # test that i got a different result with a different seed
+                env.seed(1)
+                env.set_id(0)
+                obs = env.reset()
+                chronics_outdir2 = tempfile.mkdtemp()
+                env.chronics_handler.real_data.split_and_save(start_d,
+                                                              end_d,
+                                                              chronics_outdir2)
+                maintenance_1 = pd.read_csv(os.path.join(chronics_outdir2,
+                                                         "Scenario_august_00",
+                                                         "maintenance.csv.bz2"),
+                                            sep=";").values
+                assert np.any(maintenance_0_0 != maintenance_1)
+
+                # and now test that i have the same results with the same seed
+                env.seed(0)
+                env.set_id(0)
+                obs = env.reset()
+                chronics_outdir3 = tempfile.mkdtemp()
+                env.chronics_handler.real_data.split_and_save(start_d,
+                                                              end_d,
+                                                              chronics_outdir3)
+                maintenance_0_1 = pd.read_csv(os.path.join(chronics_outdir3,
+                                                         "Scenario_august_00",
+                                                         "maintenance.csv.bz2"),
+                                              sep=";").values
+                assert np.all(maintenance_0_0 == maintenance_0_1)
+
+                # make sure i can reload the environment
+                env2 = make(os.path.join(PATH_DATA_TEST, "ieee118_R2subgrid_wcci_test_maintenance"),
+                            param=param,
+                            data_feeding_kwargs={"gridvalueClass": GridStateFromFileWithForecasts},
+                            chronics_path=chronics_outdir3)
+                env2.set_id(0)
+                obs = env2.reset()
+                # and that it has the right chroncis
+                assert np.all(env2.chronics_handler.real_data.data.maintenance.astype(dt_float) ==
+                              maintenance_0_0)
+
     def test_seed(self):
         param = Parameters()
         param.NO_OVERFLOW_DISCONNECTION = True
@@ -822,6 +891,7 @@ class TestMaintenanceBehavingNormally(HelperTests):
             with make(os.path.join(PATH_CHRONICS, "env_14_test_maintenance"),
                       test=True,
                       param=param) as env:
+                l_id = 11
                 obs = env.reset()
                 assert np.all(obs.time_before_cooldown_line == 0)
                 obs, reward, done, info = env.step(env.action_space())
@@ -829,30 +899,32 @@ class TestMaintenanceBehavingNormally(HelperTests):
                               np.array([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12, 0, 0, 0, 0, 0, 0, 0, 0], dtype=dt_int))
                 obs, reward, done, info = env.step(env.action_space({"set_line_status": [(11, 1)]}))
                 assert info["is_illegal"]
-                assert not obs.line_status[11]
+                assert not obs.line_status[l_id]
                 assert np.all(obs.time_before_cooldown_line ==
                               np.array([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 11, 0, 0, 0, 0, 0, 0, 0, 0], dtype=dt_int))
 
                 obs, reward, done, info = env.step(env.action_space({"set_bus": {"lines_or_id": [(11, 1)]}}))
-                assert info["is_illegal"] is False  # it is legal
-                assert not obs.line_status[11]  # but has no effect (line is still disconnected)
+                assert info["is_illegal"] is True  # it is legal -> no more because of cooldown (and because now this action is counted as an action on a line)
+                assert not obs.line_status[l_id]  # but has no effect (line is still disconnected)
 
                 assert np.all(obs.time_before_cooldown_line ==
                               np.array([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10, 0, 0, 0, 0, 0, 0, 0, 0], dtype=dt_int))
                 for i in range(10):
                     obs, reward, done, info = env.step(env.action_space())
                     arr_ = np.array([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], dtype=dt_int)
-                    arr_[11] = 9-i
+                    arr_[l_id] = 9-i
                     assert np.all(obs.time_before_cooldown_line == arr_), "error at time step {}".format(i)
+                # i have no more cooldown on line now
+                assert np.all(obs.time_before_cooldown_line == 0)
 
                 obs, reward, done, info = env.step(env.action_space({"set_bus": {"lines_or_id": [(11, 1)]}}))
                 assert info["is_illegal"] is False  # it is legal
-                assert not obs.line_status[11]  #  reconnecting only one end has still no effect
+                assert obs.line_status[l_id]  #  reconnecting only one end has still no effect -> change of behaviour
 
                 obs, reward, done, info = env.step(env.action_space({"set_line_status": [(11, 1)]}))
                 assert info["is_illegal"] is False  # it's legal
 
-                assert obs.line_status[11]  # it has reconnected the powerline
+                assert obs.line_status[l_id]  # it has reconnected the powerline
                 assert np.all(obs.time_before_cooldown_line ==
                               np.array([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], dtype=dt_int))
                 # nothing is in maintenance
@@ -877,6 +949,7 @@ class TestMaintenanceBehavingNormally(HelperTests):
                       test=True,
                       param=param,
                       gamerules_class=AlwaysLegal) as env:
+                l_id = 11
                 obs = env.reset()
                 assert np.all(obs.time_before_cooldown_line == 0)
                 obs, reward, done, info = env.step(env.action_space())
@@ -884,28 +957,31 @@ class TestMaintenanceBehavingNormally(HelperTests):
                               np.array([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12, 0, 0, 0, 0, 0, 0, 0, 0], dtype=dt_int))
                 obs, reward, done, info = env.step(env.action_space({"set_line_status": [(11, 1)]}))
                 assert not info["is_illegal"]  # it is legal
-                assert not obs.line_status[11]  # yet maintenance should have stayed
+                assert not obs.line_status[l_id]  # yet maintenance should have stayed
                 assert np.all(obs.time_before_cooldown_line ==
                               np.array([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 11, 0, 0, 0, 0, 0, 0, 0, 0], dtype=dt_int))
 
                 obs, reward, done, info = env.step(env.action_space({"set_bus": {"lines_or_id": [(11, 1)]}}))
                 assert not info["is_illegal"]  # it is legal
-                assert not obs.line_status[11]  # but has no effect (line is still disconnected)
+                assert not obs.line_status[l_id]  # but has no effect (line is still disconnected)
                 assert np.all(obs.time_before_cooldown_line ==
                               np.array([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10, 0, 0, 0, 0, 0, 0, 0, 0], dtype=dt_int))
                 for i in range(10):
                     obs, reward, done, info = env.step(env.action_space())
                     arr_ = np.array([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], dtype=dt_int)
-                    arr_[11] = 9-i
+                    arr_[l_id] = 9-i
                     assert np.all(obs.time_before_cooldown_line == arr_), "error at time step {}".format(i)
 
+                # i have no more cooldown on line now
+                assert np.all(obs.time_before_cooldown_line == 0)
+
                 obs, reward, done, info = env.step(env.action_space({"set_bus": {"lines_or_id": [(11, 1)]}}))
-                assert not info["is_illegal"]  # it is legal
-                assert not obs.line_status[11]  # reconnecting only one end has still no effect
+                assert not info["is_illegal"]  # it is legal -> no more because of cooldown
+                # assert not obs.line_status[11]  # reconnecting only one end has still no effect -> change of behaviour now
 
                 obs, reward, done, info = env.step(env.action_space({"set_line_status": [(11, 1)]}))
                 assert not info["is_illegal"]  # it is legal
-                assert obs.line_status[11]  # it has reconnected the powerline
+                assert obs.line_status[l_id]  # it has reconnected the powerline
                 assert np.all(obs.time_before_cooldown_line ==
                               np.array([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0], dtype=dt_int))
                 # nothing is in maintenance
