@@ -82,7 +82,21 @@ class TestLoadingBackendPandaPower(unittest.TestCase):
         pass
 
     def compare_vect(self, pred, true):
-        return dt_float(np.max(np.abs(pred- true))) <= self.tolvect
+        return dt_float(np.max(np.abs(pred - true))) <= self.tolvect
+
+    def test_copy_env(self):
+        cpy = Environment(**self.env.get_kwargs())
+        obs1 = cpy.reset()
+        obs2 = self.env.reset()
+        assert obs1 == obs2
+        obs1, reward1, done1, info1 = cpy.step(self.env.action_space())
+        obs2, reward2, done2, info2 = self.env.step(self.env.action_space())
+        assert abs(reward1 - reward2) <= self.tol_one
+        assert done1 == done2
+        assert info1.keys() == info2.keys()
+        for kk in info1.keys():
+            assert np.all(info1[kk] == info2[kk])
+        assert obs1 == obs2
 
     def test_step_doesnt_change_action(self):
         act = self.env.action_space()
@@ -293,6 +307,7 @@ class TestResetOk(unittest.TestCase):
         obs, reward, done, info = self.env.step(act)
         act = self.env.action_space.disconnect_powerline(4)
         obs, reward, done, info = self.env.step(act)
+        # at this stage there is a cascading failure
         assert len(info["exception"])
         assert isinstance(info["exception"][0], DivergingPowerFlow)
         # reset the grid
@@ -303,6 +318,37 @@ class TestResetOk(unittest.TestCase):
         simobs, simr, simdone, siminfo = obs.simulate(self.env.action_space())
         assert np.all(simobs.topo_vect == 1)
 
+    def test_reset_after_blackout_withdetailed_info(self):
+        backend = PandaPowerBackend(detailed_infos_for_cascading_failures=True)
+        with warnings.catch_warnings():
+            warnings.filterwarnings("ignore")
+            env = make("rte_case5_example", test=True, reward_class=L2RPNReward,
+                       other_rewards={"test": L2RPNReward},
+                       backend=backend)
+
+        # make the grid in bad shape
+        act = env.action_space({"set_bus": {"substations_id": [(2, [1, 2, 1, 2])]}})
+        obs, reward, done, info = env.step(act)
+        act = env.action_space({"set_bus": {"substations_id": [(0, [1, 1, 2, 2, 1, 2])]}})
+        obs, reward, done, info = env.step(act)
+        act = env.action_space({"set_bus": {"substations_id": [(3, [1, 1, 2, 2, 1])]}})
+        obs, reward, done, info = env.step(act)
+        act = env.action_space.disconnect_powerline(3)
+        obs, reward, done, info = env.step(act)
+        obs, reward, done, info = env.step(env.action_space())
+        obs, reward, done, info = env.step(env.action_space())
+        # at this stage there is a cascading failure
+        assert len(info["exception"])
+        assert isinstance(info["exception"][0], DivergingPowerFlow)
+        assert "detailed_infos_for_cascading_failures" in info
+        assert len(info["detailed_infos_for_cascading_failures"])
+        # reset the grid
+        obs = self.env.reset()
+        assert np.all(obs.topo_vect == 1)
+
+        # check that i can simulate
+        simobs, simr, simdone, siminfo = obs.simulate(self.env.action_space())
+        assert np.all(simobs.topo_vect == 1)
 
 class TestAttachLayout(unittest.TestCase):
     def test_attach(self):
