@@ -9,7 +9,7 @@
 import tempfile
 import warnings
 from grid2op.tests.helper_path_test import *
-from grid2op.Opponent import BaseOpponent, RandomLineOpponent
+from grid2op.Opponent import BaseOpponent, RandomLineOpponent, WeightedRandomOpponent
 from grid2op.Action import TopologyAction
 from grid2op.MakeEnv import make
 from grid2op.Opponent.BaseActionBudget import BaseActionBudget
@@ -24,6 +24,7 @@ import pdb
 ATTACK_DURATION = 48
 ATTACK_COOLDOWN = 100
 LINES_ATTACKED = ["1_3_3", "1_4_4", "3_6_15", "9_10_12", "11_12_13", "12_13_14"]
+RHO_NORMALIZATION = [1, 1, 1, 1, 1, 1]
 
 
 class TestSuiteBudget_001(BaseActionBudget):
@@ -45,6 +46,21 @@ class TestSuiteOpponent_001(BaseOpponent):
         return attack
 
 
+class TestWeightedRandomOpponent(WeightedRandomOpponent):
+    def init(self, lines_attacked=[], rho_normalization=[], **kwargs):
+        WeightedRandomOpponent.init(self, lines_attacked=lines_attacked, rho_normalization=rho_normalization, **kwargs)
+        self._attack_counter = 0
+        self._attack_continues_counter = 0
+
+    def tell_attack_continues(self, observation, agent_action, env_action, budget):
+        self._attack_continues_counter += 1
+        WeightedRandomOpponent.tell_attack_continues(self, observation, agent_action, env_action, budget)
+
+    def attack(self, observation, agent_action, env_action, budget, previous_fails):
+        self._attack_counter += 1
+        return WeightedRandomOpponent.attack(self, observation, agent_action, env_action, budget, previous_fails)
+
+
 class TestLoadingOpp(unittest.TestCase):
     def test_creation_BaseOpponent(self):
         with warnings.catch_warnings():
@@ -715,6 +731,382 @@ class TestLoadingOpp(unittest.TestCase):
         env.close()
         multi_env.close()
 
+    def test_WeightedRandomOpponent_not_enough_budget(self):
+        """Tests that the attack is ignored when the budget is too low"""
+        with warnings.catch_warnings():
+            warnings.filterwarnings("ignore")
+            init_budget = 50
+            with make("rte_case14_realistic",
+                      test=True,
+                      opponent_attack_cooldown=1,  # only for testing
+                      opponent_init_budget=init_budget,
+                      opponent_budget_per_ts=0.,
+                      opponent_action_class=TopologyAction,
+                      opponent_budget_class=BaseActionBudget,
+                      opponent_attack_duration=ATTACK_DURATION,
+                      opponent_class=WeightedRandomOpponent,
+                      kwargs_opponent={"lines_attacked": LINES_ATTACKED,
+                                       "rho_normalization": RHO_NORMALIZATION,
+                                       "attack_period": 1}) as env:
+                env.seed(0)
+                obs = env.reset()
+                assert env.oppSpace.budget == init_budget
+                # The opponent can attack
+                for i in range(env.oppSpace.attack_duration):
+                    obs, reward, done, info = env.step(env.action_space())
+                    attack = env.oppSpace.last_attack
+                    assert env.oppSpace.budget == init_budget - i - 1
+                    assert any(attack._set_line_status != 0)
+
+                # There is not enough budget for a second attack
+                assert abs(env.oppSpace.budget - (init_budget - ATTACK_DURATION)) <= 1e-5
+                obs, reward, done, info = env.step(env.action_space())
+                attack = env.oppSpace.last_attack
+                assert attack is None
+
+    def test_WeightedRandomOpponent_attackable_lines(self):
+        """Tests that the WeightedRandomOpponent only attacks the authorized lines"""
+        with warnings.catch_warnings():
+            warnings.filterwarnings("ignore")
+            init_budget = 1000
+            tries = 30
+            attackable_lines_case14 = LINES_ATTACKED
+            with make("rte_case14_realistic",
+                      test=True,
+                      opponent_init_budget=init_budget,
+                      opponent_budget_per_ts=0.,
+                      opponent_action_class=TopologyAction,
+                      opponent_budget_class=BaseActionBudget,
+                      opponent_attack_duration=ATTACK_DURATION,
+                      opponent_attack_cooldown=ATTACK_COOLDOWN,
+                      opponent_class=WeightedRandomOpponent,
+                      kwargs_opponent={"lines_attacked": LINES_ATTACKED,
+                                       "rho_normalization": RHO_NORMALIZATION,
+                                       "attack_period": 1}) as env:
+                env.seed(0)
+                # Collect some attacks and check that they belong to the correct lines
+                for _ in range(tries):
+                    obs = env.reset()
+                    assert env.oppSpace.budget == init_budget
+                    obs, reward, done, info = env.step(env.action_space())
+                    assert env.oppSpace.budget == init_budget - 1
+
+                    attack = env.oppSpace.last_attack
+                    attacked_line = np.where(attack._set_line_status == -1)[0][0]
+                    line_name = env.action_space.name_line[attacked_line]
+                    assert line_name in attackable_lines_case14
+
+    def test_WeightedRandomOpponent_disconnects_only_one_line(self):
+        """Tests that the WeightedRandomOpponent does not disconnect several lines at a time"""
+        with warnings.catch_warnings():
+            warnings.filterwarnings("ignore")
+            init_budget = 1000
+            tries = 30
+            with make("rte_case14_realistic",
+                      test=True,
+                      opponent_init_budget=init_budget,
+                      opponent_budget_per_ts=0.,
+                      opponent_action_class=TopologyAction,
+                      opponent_budget_class=BaseActionBudget,
+                      opponent_attack_duration=ATTACK_DURATION,
+                      opponent_attack_cooldown=ATTACK_COOLDOWN,
+                      opponent_class=WeightedRandomOpponent,
+                      kwargs_opponent={"lines_attacked": LINES_ATTACKED,
+                                       "rho_normalization": RHO_NORMALIZATION,
+                                       "attack_period": 1}) as env:
+                env.seed(0)
+                # Collect some attacks and check that they belong to the correct lines
+                for _ in range(tries):
+                    obs = env.reset()
+                    assert env.oppSpace.budget == init_budget
+                    obs, reward, done, info = env.step(env.action_space())
+                    assert env.oppSpace.budget == init_budget - 1
+
+                    attack = env.oppSpace.last_attack
+                    n_disconnected = np.sum(attack._set_line_status == -1)
+                    assert n_disconnected == 1
+
+    def test_WeightedRandomOpponent_with_agent(self):
+        """Tests that the line status cooldown is correctly updated when the opponent attacks a line with an agent"""
+        with warnings.catch_warnings():
+            warnings.filterwarnings("ignore")
+            # to prevent attack on first time step
+            init_budget = 8
+            opponent_budget_per_ts = 1
+            length = 300
+            agent_line_cooldown = 30
+            attack_duration = 10
+            attack_cooldown = 20000  # i do one attack
+            param = Parameters()
+            param.NO_OVERFLOW_DISCONNECTION = True
+            param.NB_TIMESTEP_COOLDOWN_LINE = agent_line_cooldown
+            line_opponent_attack = 4
+            line_opponent_attack = 15
+            lines_attacked = ["3_6_15"]
+            rho_normalization = [1]
+            with make("rte_case14_realistic",
+                      test=True, param=param,
+                      opponent_init_budget=init_budget,
+                      opponent_budget_per_ts=opponent_budget_per_ts,
+                      opponent_action_class=TopologyAction,
+                      opponent_budget_class=BaseActionBudget,
+                      opponent_attack_duration=attack_duration,
+                      opponent_attack_cooldown=attack_cooldown,
+                      opponent_class=WeightedRandomOpponent,
+                      kwargs_opponent={"lines_attacked": lines_attacked,
+                                       "rho_normalization": rho_normalization,
+                                       "attack_period": 1}) as env:
+                env.seed(0)
+                obs = env.reset()
+                reward = 0
+                assert env.oppSpace.budget == init_budget
+                assert np.all(obs.time_before_cooldown_line == 0)
+                # the "agent" does an action (on the same powerline as the opponent attacks)
+                obs, reward, done, info = env.step(env.action_space({"set_line_status": [(line_opponent_attack, 1)]}))
+                assert np.all(obs.line_status)
+                assert obs.time_before_cooldown_line[line_opponent_attack] == agent_line_cooldown
+
+                # check that the opponent cooldown is not taken into account (lower than the cooldown on line)
+                for i in range(10):
+                    obs, reward, done, info = env.step(env.action_space())
+                    assert "opponent_attack_line" in info
+                    assert np.sum(info["opponent_attack_line"]) == 1, "error at iteration {} for attack".format(i)
+                    assert info["opponent_attack_line"][line_opponent_attack]
+                    assert obs.time_before_cooldown_line[line_opponent_attack] == agent_line_cooldown - (i+1), "error at iteration {}".format(i)
+
+                obs, reward, done, info = env.step(env.action_space())
+                assert "opponent_attack_line" in info
+                assert info["opponent_attack_line"] is None  # no more attack
+                assert obs.time_before_cooldown_line[line_opponent_attack] == agent_line_cooldown - 11
+
+    def test_WeightedRandomOpponent_with_maintenance_1(self):
+        """Tests that the line status cooldown is correctly updated when the opponent attacks a line with an agent"""
+
+        with warnings.catch_warnings():
+            warnings.filterwarnings("ignore")
+            # to prevent attack on first time step
+            init_budget = 8
+            opponent_budget_per_ts = 1
+            length = 300
+            agent_line_cooldown = 30
+            attack_duration = 5
+            attack_cooldown = 20000  # i do one attack
+            param = Parameters()
+            param.NO_OVERFLOW_DISCONNECTION = True
+            param.NB_TIMESTEP_COOLDOWN_LINE = agent_line_cooldown
+            line_opponent_attack = 4
+            line_opponent_attack = 11
+
+            # 1. attack is at the same time than the maintenance
+            lines_attacked = ["8_13_11"]
+            rho_normalization = [1]
+            with make(os.path.join(PATH_CHRONICS, "env_14_test_maintenance"),
+                      test=True, param=param,
+                      opponent_init_budget=init_budget,
+                      opponent_budget_per_ts=opponent_budget_per_ts,
+                      opponent_action_class=TopologyAction,
+                      opponent_budget_class=BaseActionBudget,
+                      opponent_attack_duration=attack_duration,
+                      opponent_attack_cooldown=attack_cooldown,
+                      opponent_class=WeightedRandomOpponent,
+                      kwargs_opponent={"lines_attacked": lines_attacked,
+                                       "rho_normalization": rho_normalization,
+                                       "attack_period": 1}) as env:
+                env.seed(0)
+                obs = env.reset()
+                obs, reward, done, info = env.step(env.action_space())
+                # the opponent has attacked
+                assert "opponent_attack_line" in info
+                assert np.sum(info["opponent_attack_line"]) == 1, "error at iteration 0 for attack"
+                assert info["opponent_attack_line"][line_opponent_attack]
+                # but the maintenance cooldown has priority (longer)
+                assert np.all(obs.time_before_cooldown_line ==
+                              np.array([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12, 0, 0, 0, 0, 0, 0, 0, 0], dtype=dt_int))
+
+            # 2. attack is before than the maintenance
+            init_budget = 8
+            opponent_budget_per_ts = 1
+            attack_duration = 5
+            lines_attacked = ["9_10_12"]
+            rho_normalization = [1]
+            line_id = 12
+            with make(os.path.join(PATH_CHRONICS, "env_14_test_maintenance"),
+                      test=True, param=param,
+                      opponent_init_budget=init_budget,
+                      opponent_budget_per_ts=opponent_budget_per_ts,
+                      opponent_action_class=TopologyAction,
+                      opponent_budget_class=BaseActionBudget,
+                      opponent_attack_duration=attack_duration,
+                      opponent_attack_cooldown=attack_cooldown,
+                      opponent_class=WeightedRandomOpponent,
+                      kwargs_opponent={"lines_attacked": lines_attacked,
+                                       "rho_normalization": rho_normalization,
+                                       "attack_period": 1}) as env:
+                env.seed(0)
+                obs = env.reset()
+                env.fast_forward_chronics(274)
+                obs, reward, done, info = env.step(env.action_space())
+                # i have a maintenance in 1 time step
+                assert obs.time_next_maintenance[line_id] == 1
+                # the opponent has attacked at this time step
+                assert "opponent_attack_line" in info
+                assert info["opponent_attack_line"] is not None
+                assert info["opponent_attack_line"][line_id]
+                assert info["opponent_attack_duration"] == 4
+                # cooldown should be updated correctly
+                assert np.all(obs.time_before_cooldown_line ==
+                              np.array([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 3, 0, 0, 0, 0, 0, 0, 0], dtype=dt_int))
+                for i in range(3):
+                    obs, reward, done, info = env.step(env.action_space())  # the maintenance is happening
+                    # i have a maintenance in 1 time step
+                    assert obs.time_next_maintenance[line_id] == 0
+                    # the attack continued
+                    assert "opponent_attack_line" in info
+                    assert info["opponent_attack_line"] is not None
+                    assert info["opponent_attack_line"][line_id]
+                    assert info["opponent_attack_duration"] == 3-i
+                    assert np.all(obs.time_before_cooldown_line ==
+                                  np.array([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12-i, 0, 0, 0, 0, 0, 0, 0], dtype=dt_int))
+
+                # attack should be over
+                obs, reward, done, info = env.step(env.action_space())  # the maintenance is happening
+                # i have a maintenance in 1 time step
+                assert obs.time_next_maintenance[line_id] == 0
+                # the attack continued
+                assert "opponent_attack_line" in info
+                assert info["opponent_attack_line"] is None
+                assert info["opponent_attack_duration"] == 0
+                assert np.all(obs.time_before_cooldown_line ==
+                              np.array([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 12-3, 0, 0, 0, 0, 0, 0, 0], dtype=dt_int))
+
+    def test_WeightedRandomOpponent_only_attack_connected(self):
+        """
+        Tests that the WeightedRandomOpponent does not attack lines that are already disconnected
+        """
+        with warnings.catch_warnings():
+            warnings.filterwarnings("ignore")
+            init_budget = 10000
+            length = 300
+            env = make("rte_case14_realistic",
+                       test=True,
+                       opponent_init_budget=init_budget,
+                       opponent_budget_per_ts=0.,
+                       opponent_attack_cooldown=1, # only for testing
+                       opponent_action_class=TopologyAction,
+                       opponent_budget_class=BaseActionBudget,
+                       opponent_attack_duration=ATTACK_DURATION,
+                       opponent_class=WeightedRandomOpponent,
+                      kwargs_opponent={"lines_attacked": LINES_ATTACKED,
+                                       "rho_normalization": RHO_NORMALIZATION,
+                                       "attack_period": 1})
+            env.seed(0)
+            # Collect some attacks
+            # and check that they belong to the correct lines
+            pre_obs = env.reset()
+            done = False
+            assert env.oppSpace.budget == init_budget
+            for i in range(length):
+                obs, reward, done, info = env.step(env.action_space())
+                    
+                attack = env.oppSpace.last_attack
+                attacked_line = np.where(attack._set_line_status == -1)[0][0]
+                if env.oppSpace.current_attack_duration < env.oppSpace.attack_duration:
+                    # The attack is ungoing. The line must have been disconnected already
+                    assert not pre_obs.line_status[attacked_line]
+                else:
+                    # A new attack was launched. The line must have been connected
+                    assert pre_obs.line_status[attacked_line]
+                    
+                pre_obs = obs
+                if done:
+                    pre_obs = env.reset()
+
+    def test_WeightedRandomOpponent_same_attack_order_and_attacks_all_lines(self):
+        """Tests that the WeightedRandomOpponent has the same attack order (when seeded) and attacks all lines"""
+        with warnings.catch_warnings():
+            warnings.filterwarnings("ignore")
+            init_budget = 1000
+            length = 30
+            expected_attack_order = [
+                    4, 3, 14, 15, 12,
+                    4, 3, 15, 12, 13,
+                    12, 4, 3, 13, 14,
+                    12, 3, 3, 4, 15,
+                    13, 14, 12, 4, 3,
+                    15, 13, 14, 4]
+
+            attack_order = []
+            has_disconnected_all = False
+            with make("rte_case14_realistic",
+                      test=True,
+                      opponent_init_budget=init_budget,
+                      opponent_budget_per_ts=0.,
+                      opponent_attack_cooldown=1,  # only for testing
+                      opponent_attack_duration=1,  # only for testing
+                      opponent_action_class=TopologyAction,
+                      opponent_budget_class=BaseActionBudget,
+                      opponent_class=WeightedRandomOpponent,
+                      kwargs_opponent={"lines_attacked": LINES_ATTACKED,
+                                       "rho_normalization": RHO_NORMALIZATION,
+                                       "attack_period": 1}) as env:
+                env.seed(0)
+                # Collect some attacks and check that they belong to the correct lines
+                obs = env.reset()
+                done = False
+                assert env.oppSpace.budget == init_budget
+                for i in range(length):
+                    if done:
+                        obs = env.reset()
+                    pre_done = done
+                    obs, reward, done, info = env.step(env.action_space())
+
+                    attack = env.oppSpace.last_attack
+                    if attack is None: # should only happen here if all attackable lines are already disconnected
+                        assert np.sum(obs.line_status == False) == 6
+                        continue
+
+                    assert any(attack._set_line_status == -1)
+                    attacked_line = np.where(attack._set_line_status == -1)[0][0]
+                    if pre_done or not (attack_order and attack_order[-1] == attacked_line):
+                        attack_order.append(attacked_line)
+
+                assert attack_order == expected_attack_order
+                assert len(set(attack_order)) == 6
+
+    def test_either_attack_or_tell_attack_continues(self):
+        """Tests that at each step, either the attack or the tell_attack_continues method is called once"""
+        with warnings.catch_warnings():
+            warnings.filterwarnings("ignore")
+            init_budget = 1000
+            length = 100
+            attack_cooldown = 15
+            with make("rte_case14_realistic",
+                      test=True,
+                      opponent_init_budget=init_budget,
+                      opponent_budget_per_ts=0.,
+                      opponent_attack_cooldown=attack_cooldown,  # only for testing
+                      opponent_attack_duration=5,  # only for testing
+                      opponent_action_class=TopologyAction,
+                      opponent_budget_class=BaseActionBudget,
+                      opponent_class=TestWeightedRandomOpponent,
+                      kwargs_opponent={"lines_attacked": LINES_ATTACKED,
+                                       "rho_normalization": RHO_NORMALIZATION,
+                                       "attack_period": attack_cooldown}) as env:
+                env.seed(0)
+                # Collect some attacks and check that they belong to the correct lines
+                obs = env.reset()
+                done = False
+                assert env.oppSpace.budget == init_budget
+                for i in range(length):
+                    if done:
+                        obs = env.reset()
+                    obs, reward, done, info = env.step(env.action_space())
+                assert env.oppSpace.opponent._attack_counter == 56
+                assert env.oppSpace.opponent._attack_continues_counter == 44
+                assert env.oppSpace.opponent._attack_counter \
+                     + env.oppSpace.opponent._attack_continues_counter \
+                       == length
+
 
 if __name__ == "__main__":
     unittest.main()
